## Kubectl

### Syntax

`kubectl`, tüm operasyonlar için aşağıdaki formda ortak bir söz dizimi kullanır:

```
$ kubectl <command> <type> <name> <flags>
```

* **command** -  Gerçekleştirilecek komut veya operasyon. örn. apply, create, delete, ve get.
* **type** - Kaynak türü veya nesne.
* **name** - Kaynağın veya nesnenin adı.
* **flags** - Komuta iletilecek isteğe bağlı bayraklar (flags).

**Examples**
```
$ kubectl create -f examples/nginx-pod.yaml
$ kubectl get pods
$ kubectl get pod nginx
$ kubectl delete pod nginx
```

`kubectl`, bir kullanıcının birden fazla Kubernetes cluster'ı ile etkileşim kurmasına ve yönetmesine olanak tanır. Bunu yapmak için, **context** (bağlam) olarak bilinen şeye ihtiyaç duyar. Bir context; `cluster`, `namespace` ve `user` kombinasyonundan oluşur.

* **cluster** - Kubernetes cluster'ı için kolay bir ad, sunucu adresi ve sertifika.

* **namespace (isteğe bağlı)** - Kullanılacak mantıksal cluster veya ortam. Eğer belirtilmezse, varsayılan
    `default` namespace kullanılır.

* **user** - Cluster'a bağlanmak için kullanılan kimlik bilgileri. Bu, client sertifikası ve anahtarı,
    kullanıcı adı/şifre veya token kombinasyonu olabilir.

Bu context'ler, **`kubeconfig`** olarak adlandırılan yerel yaml tabanlı bir yapılandırma (config) dosyasında saklanır. \*nix tabanlı sistemler için `kubeconfig`, `$HOME/.kube/config` içinde saklanır; Windows için `%USERPROFILE%/.kube/config` yolunda bulunabilir.

Bu yapılandırma, dosyayı doğrudan görüntülemeye gerek kalmadan görüntülenebilir.

**command**
```
$ kubectl config view
```

---

### kubectl Basics
Her türlü günlük operasyon için sıkça kullanılan birkaç `kubectl` komutu vardır: `get`, `create`, `apply`, `delete`, `describe`, ve `logs`. Diğer komutlar basitçe `kubectl --help` veya `kubectl <command> --help` ile listelenebilir.

---

### kubectl get
`kubectl get`, belirli bir türdeki nesneleri veya belirli bir nesnenin kendisini getirir ve listeler. Ayrıca, `-o` veya `--output` bayrağı aracılığıyla bilgiyi json, yaml, wide (ek sütunlar) veya name (sadece adlar) gibi çeşitli kullanışlı formatlarda yazdırmayı destekler.

**Command**
```
$ kubectl get <type>
$ kubectl get <type> <name>
$ kubectl get <type> <name> -o <output format>
```

**Examples**
```
$ kubectl get namespaces

$ kubectl get pod mypod -o wide
```

---

### kubectl create
`kubectl create`, komut satırından (`stdin`) veya sağlanan bir json/yaml manifest dosyasından bir nesne oluşturur. Manifest'ler, `-f` veya `--filename` bayrağı ile belirtilebilir; bu bayrak ya bir dosyayı ya da birden fazla manifest içeren bir dizini gösterebilir.

**Command**
```
$ kubectl create <type> <parameters>
$ kubectl create -f <path to manifest>
```

**Examples**
```
$ kubectl create namespace dev

$ kubectl create -f examples/nginx-pod.yaml
```
---
### kubectl apply
`kubectl apply`, `kubectl create` komutuna benzer. Eğer kaynak zaten oluşturulmuşsa onu günceller, eğer henüz mevcut değilse basitçe oluşturur. Yapılandırmayı güncellediğinde, önceki sürümünü oluşturulan nesnenin kendisinde bir **`annotation`** (ek açıklama) içinde saklar.

 > **UYARI:** Eğer nesne başlangıçta `kubectl apply` ile oluşturulmadıysa, güncelleme davranışı iki yönlü bir fark (diff) olarak hareket edecektir. Bu konuda daha fazla bilgi için lütfen [kubectl apply](https://kubernetes.io/docs/concepts/cluster-administration/manage-deployment/#kubectl-apply) belgelerine bakın.

*   Tıpkı `kubectl create` gibi, `-f` bayrağı ile bir json veya yaml dosyası alır veya `stdin`'den girdi kabul eder.

**Command**
```
$ kubectl apply -f <path to yaml>
```

**Examples**
```
$ kubectl apply -f examples/nginx-pod.yaml
$ kubectl apply -f examples/svc.yaml
```

---

### kubectl edit
`kubectl edit`, güncellenmiş bir manifest uygulamak zorunda kalmadan bir kaynağı yerinde değiştirir. İstenen nesnenin bir kopyasını getirir ve `KUBE_EDITOR` veya `EDITOR` ortam değişkenleri tarafından ayarlanan yapılandırılmış metin düzenleyici ile yerel olarak açar. Bu komut sorun giderme için kullanışlıdır, ancak değişiklikler esasen takip edilemez olacağından üretim senaryolarında kaçınılmalıdır.


**Command**
```
$ kubectl edit <type> <object name>
```

**Examples**
```
$ kubectl edit pod nginx
$ kubectl edit service service-example
```

---

### kubectl delete
`kubectl delete`, nesneyi Kubernetes'ten siler.

**Command**
```
$ kubectl delete <type> <name>
```

**Examples**
```
$ kubectl delete pod nginx
```

---

### kubectl describe
`kubectl describe`, belirli Kubernetes nesnesi hakkında ayrıntılı bilgi listeler. Çok yardımcı bir sorun giderme aracıdır.

**Command**
```
$ kubectl describe <type>
$ kubectl describe <type> <name>
```

**Examples**
```
$ kubectl describe pod nginx
```

### kubectl logs
`kubectl logs`, bir pod'dan birleştirilmiş `stdout` ve `stderr` log'larını yazdırır. Bir `pod` içinde birden fazla container varsa, `-c` bayrağı kullanılır ve container adı belirtilmelidir.

**Command**
```
$ kubectl logs <pod name>
$ kubectl logs <pod name> -c <container name>
```

**Examples**
```
$ kubectl logs nginx
```

---

### kubectl exec
`kubectl exec`, bir Pod içinde bir komut çalıştırır ve isteğe bağlı olarak uzak bir container içinde etkileşimli bir terminal başlatabilir. Bir Pod içinde birden fazla container bulunduğunda, `-c` veya `--container` bayrağı ve ardından container adı gereklidir.

Etkileşimli bir oturum isteniyorsa, `-i` (`--stdin`) ve `-t` (`--tty`) bayrakları sağlanmalıdır.

**Command**
```
$ kubectl exec <pod name> -- <arg>
$ kubectl exec <pod name> -c <container name> -- <arg>
$ kubectl exec  -i -t <pod name> -c <container name> -- <arg>
$ kubectl exec  -it <pod name> -c <container name> -- <arg>
```
---
## YAML Dosyası

`kubectl`,  **.yaml** dosyası ile API isteği yaparken bilgiyi **JSON**'a dönüştürür.


### Alıştırma: Bir YAML dosyasından Nesne (Object) Oluşturma

**Hedefler:** `kubectl` kullanarak Kubernetes Nesneleri'nin (Objects) nasıl oluşturulacağını öğrenin.

1)  `kubectl` kullanarak Deployment'ı oluşturun:

    ```
    $ kubectl apply -f examples/deployment.yaml
    ```

2)  Birkaç saniye sonra deployment'ları listeleyin. Cluster'da oluşturulan deployment nesnesini göreceksiniz.

    ```
    $ kubectl get deployments
    ```

3)  Daha fazla bilgi için deployment'ı describe edin (detaylarını gösterin).

    ```
    $ kubectl describe deployments nginx-deployment
    ```
**Özet**: `kubectl create` komutu, `-f` bayrağı ile birlikte kullanıldı; bu, API'yi çağırır ve ona bunun bir dosyadan oluşturulacağını söyler.