## Namespaces
Namespaces, mantıksal bir cluster (küme) veya ortamdır. Bir cluster'ı bölümlere ayırmanın (partitioning) veya erişimi kapsamlandırmanın (scoping access) birincil yöntemidir.

### Alıştırma: Namespaces Kullanımı
**Hedefler:** `kubectl` kullanarak Kubernetes Namespaces'lerinin nasıl oluşturulacağını ve aralarında nasıl geçiş yapılacağını öğrenin.



1) Mevcut namespace'leri listeleyin:
```
$ kubectl get namespaces
```

2) `dev` namespace'ini oluşturun:
```
$ kubectl create namespace dev
```

**Özet:** Namespaces, kapsamlı adlar (scoped names), erişim sağlamanın birincil yöntemi olarak işlev görür ve grup tabanlı kaynak kısıtlamaları için bir şemsiye görevi görür. Bunları oluşturmak ve aralarında geçiş yapmak hızlı ve kolaydır, ancak bunları kullanmayı öğrenmek Kubernetes'in genel kullanımında esastır.

 ## Pods

**Hedefler:** `kubectl` ve `yaml` kullanarak Kubernetes Pod'larının nasıl oluşturulacağını öğrenin.

Bir pod, paylaşılan depolama/ağ (shared storage/network) ve container'ların nasıl çalıştırılacağına dair bir belirtim (specification) ile birlikte bir veya daha fazla container grubudur. Docker, bir Kubernetes Pod'unda kullanılan en yaygın container runtime'ıdır (çalışma zamanı), ancak Pod'lar diğer container runtime'larını da destekler.


**Hedef:** Tek container içeren Pod'u inceleyin.

1) `nginx` imajını kullanarak ve `80` portunu açarak (expose) `nginx` adında basit bir Pod oluşturun. `examples/pod.yaml` dosyasını veya aşağıdaki yaml kodunu kullanın.

**examples/pod.yaml**
```yaml
apiVersion: v1
kind: Pod
metadata:
  name: nginx
spec:
  containers:
    - name: nginx
      image: nginx
      ports:
        - containerPort: 80
```

**Command**
```
$ kubectl apply -f examples/pod.yaml
```

2) Pod'u describe etmek (detaylarını göstermek) ve mevcut bilgileri not etmek için `kubectl` kullanın.
```
$ kubectl describe pod nginx
```

**Özet:** Bir Pod'un genel yönlerini oluşturma ve görüntüleme konusunda aşinalık kazanmak önemli bir beceridir. Pod'lar bir YAML dosyasından oluşturulur.

## Multi-Container Pod
### Init Container
Bir pod içerisinde uygulamaları çalıştıran birden fazla konteyner olabileceği gibi, uygulama konteynerleri başlatılmadan önce çalıştırılan bir veya daha fazla init konteyneri de olabilir.

### Alıştırma: YAML'dan İnit Kontaynerlı Pod Oluşturma

**Hedefler:** `kubectl` kullanarak init kontayner'lerin nasıl oluşturulacağını öğrenin.

1. Klasörünüzde `init-pod.yaml` adında bir dosya oluşturun.

2. Aşağıdaki YAML dosyasını kopyalayın veya `examples/init-pod.yaml` dosyasını kullanın.

**examples/init-pod.yaml**
```yaml
apiVersion: v1
kind: Pod
metadata:
  name: myapp-pod
  labels:
    app.kubernetes.io/name: MyApp
spec:
  containers:
  - name: myapp-container
    image: busybox:1.28
    command: ['sh', '-c', 'echo The app is running! && sleep 3600']
  initContainers:
  - name: init-myservice
    image: busybox:1.28
    command: ['sh', '-c', "until nslookup kubernetes.$(cat /var/run/secrets/kubernetes.io/serviceaccount/namespace).svc.cluster.local; do echo waiting for myservice; sleep 2; done"]
```

3. ` kubectl ` kullanarak pod oluşturun
```
$ kubectl apply -f examples/init-pod.yaml
```
4. ` kubectl `kullanarak pod'u kontrol edin:
```
$ kubectl get pods
```
----
### Sidecar Container
Kubernetes, sidecar kontaynerlarına init kontaynerlarından farklı özel bir durumu olarak uygular ; sidecar kontaynerları Pod başlatıldıktan sonra çalışmaya devam eder. 

### Alıştırma: YAML'dan İnit Kontaynerlı Pod Oluşturma

**Hedefler:** `kubectl` kullanarak sidecar kontayner'lerin nasıl oluşturulacağını öğrenin.

1. Klasörünüzde `side-pod.yaml` adında bir dosya oluşturun.

2. Aşağıdaki YAML dosyasını kopyalayın veya `examples/side-pod.yaml` dosyasını kullanın.

**examples/side-pod.yaml**
```yaml
apiVersion: v1
kind: Pod
metadata:
  name: pod-with-sidecar
spec:
  volumes:
  - name: shared-logs 
    emptyDir: {}
  containers:
  - name: app-container
    image: alpine
    command: ["/bin/sh"]
    args: ["-c", "while true; do date >> /var/log/app.txt; sleep 5;done"]
    volumeMounts:
    - name: shared-logs
      mountPath: /var/log
  # Sidecar container
  - name: sidecar-container
    image: nginx
    ports:
      - containerPort: 80
    volumeMounts:
    - name: shared-logs
      mountPath: /usr/share/nginx/html 
```

3. ` kubectl ` kullanarak pod oluşturun

```
$ kubectl apply -f examples/side-pod.yaml
```
4. ` kubectl `kullanarak pod'u kontrol edin:

```
$ kubectl get pods

```
# ReplicaSet

* Pod replikalarının ve yaşam döngülerinin (lifecycle) yönetilmesinin birincil yöntemidir. Görevleri basittir: Her zaman istenen sayıda pod'un çalışır durumda olmasını sağlamak.


### Alıştırma: YAML'dan ReplicaSet Oluşturma

**Hedefler:** `kubectl` kullanarak ReplicaSet'lerin nasıl oluşturulacağını öğrenin.

1. Klasörünüzde `replica.yaml` adında bir dosya oluşturun.

2. Aşağıdaki YAML dosyasını kopyalayın veya `examples/replica.yaml` dosyasını kullanın.

**examples/replicaset.yaml**
```yaml
apiVersion: apps/v1
kind: ReplicaSet
metadata:
 name: replicaset-example
 labels:
   app: my-app
   version: v1
spec:
 replicas: 3
 selector:
   matchLabels:
     app: my-app
 template:
   metadata:
     labels:
       app: my-app
   spec:
     containers:
     - name: nginx-app
       image: nginx:stable-alpine
       ports:
       - containerPort: 80
```

3. ` kubectl ` kullanarak ReplicaSet oluşturun

```
$ kubectl apply -f examples/replicaset.yaml
```
4. ` kubectl `kullanarak ReplicaSet'i kontrol edin:

```
$ kubectl get replicasets

```

5. ` kubectl `  kullanarak Pod'ları kontrol edin:
```
$ kubectl get pods
```
----
**Özet**: Bir `.yaml` dosyasından `ReplicaSet` oluşturmak, `“replicaset-example”` Pod'unun 3 örneğini (instance) oluşturur ve adlarını benzersiz hale getirir.

# Deployment

* Deployment, ReplicaSet'leri yöneten ve Pod'lara bildirimsel güncellemeler (declarative updates) ile birlikte birçok diğer kullanışlı özelliği sağlayan daha üst düzey bir konsepttir.

### Alıştırma: YAML'dan Deployment Oluşturma

**Hedefler:** `kubectl` kullanarak Deployment'ın nasıl oluşturulacağını öğrenin.

1. Klasörünüzde `deployment.yaml` oluşturun veya `examples/deployment.yaml` dosyasını kullanın.

**examples/deployment.yaml**
```yaml
apiVersion: apps/v1
kind: Deployment
metadata:
  name: nginx-deployment
  labels:
    app: nginx
spec:
  replicas: 3
  selector:
    matchLabels:
      app: nginx
  template:
    metadata:
      labels:
        app: nginx
    spec:
      containers:
      - name: nginx
        image: nginx:1.14.2
        ports:
        - containerPort: 80
```
3. `kubectl` kullanarak Deployment oluşturun:
```
$ kubectl apply -f examples/deployment.yaml
```
4. Deployment'ın durumunu kontrol edin:
```
$ kubectl get deployments
```
5. Deployment hazır olduğunda, mevcut ReplicaSet'leri görüntüleyin ve etiketleri (labels) gösterdiğinizden emin olun:
```
$ kubectl get rs --show-labels
```
* Yeni oluşturulan ReplicaSet'in adını ve `pod-template-hash` etiketini not edin. Oluşturulan ReplicaSet'in adı `pod-template-hash`'i içerecektir.

6. Oluşturulan ReplicaSet'i describe edin (detaylarını gösterin):
```
$ kubectl describe rs nginx-deployment-<pod-template-hash>
```
* Hem `Labels` hem de `Selectors` alanlarına bakın. `pod-template-hash` değeri otomatik olarak ReplicaSet'in hem Etiketlerine hem de Seçicisine eklenmiştir. Ardından `Controlled By` alanını not edin. Bu, doğrudan üst nesneye referans verecektir ve bu durumda orijinal `nginx-deployment` Deployment'ıdır.

7. Şimdi, Pod'ları alın ve `--show-labels` bayrağını ekleyin:
```
$ kubectl get pods --show-labels
```
* Tıpkı ReplicaSet'te olduğu gibi, Pod'ların adı ve etiketleri `pod-template-hash`'i içerir.

8. Pod'lardan birini describe edin:
```
$ kubectl describe pod nginx-deployment-<pod-template-hash-<random>
```
* `Controlled By` alanına bakın. Üst ReplicaSet'e bir referans içerecektir, ancak üst Deployment'a değil.
Artık Deployment'tan ReplicaSet'e ve Pod'a olan ilişki anlaşıldı. Şimdi `nginx-deployment`'ı güncelleme ve bir güncellemeyi iş başında görme zamanı.

9. `nginx-deployment` manifest dosyasını güncelleyin ve Pod şablonuna (template) birkaç ek etiket ekleyin. Tamamlandığında, değişikliği uygulayın:
```
$ kubectl edit deploy nginx-deployment 
```
* Pod'ların eski sürümü teker teker aşamalı olarak kaldırılacak ve yeni sürümün örnekleri yerini alacaktır. Bunun kontrol edilme şekli `strategy` bölümü aracılığıyladır.

10. Şimdi ReplicaSet'leri görüntüleyin:
```
$ kubectl get rs --show-labels
```
* Artık iki ReplicaSet olacak ve Deployment'ın önceki sürümü 0'a ölçeklendirilmiş (scaled down) olacaktır.

11. Şimdi, Deployment'ı bir ReplicaSet gibi ölçeklendirin (scale up) ve `replicas=5` olarak ayarlayın:
```
$ kubectl scale deploy nginx-deployment --replicas=5
``` 

12. ReplicaSet'leri tekrar listeleyin:
```
$ kubectl get rs --show-labels
```
* Yeni bir ReplicaSet oluşturulmadığına dikkat edin. Ölçeklendirme eylemleri Pod Şablonunda bir değişikliği tetiklemez (trigger).

### Deployment Özellikleri
* RevisionHistoryLimit
* Strategy
  * Recreate
  * RollingUpdate

### Liveness Ve Readiness Probe

#### Alıştırma: YAML'dan Deployment İçerisinde Liveness ve Readiness Probe Oluşturun

**Hedefler:** `kubectl` kullanarak probeların nasıl oluşturulacağını öğrenin.

1. Klasörünüzde `deploy-live.yaml` adında bir dosya oluşturun.

2. Aşağıdaki YAML dosyasını kopyalayın veya `examples/deploy-live.yaml` dosyasını kullanın.

**examples/deploy-live.yaml**
```yaml
apiVersion: apps/v1
kind: Deployment
metadata:
  name: nginx-deployment
spec:
  replicas: 2
  selector:
    matchLabels:
      app: nginx-deployment-web
  template:
    metadata:
      labels:
        app: nginx-deployment-web
    spec:
      containers:
        - name: c1
          image: nginx:latest
          ports:
            - containerPort: 80
          readinessProbe:
            httpGet:
              path: /
              port: 80
            initialDelaySeconds: 15
            periodSeconds: 10
          livenessProbe:
            httpGet:
              path: /hello
              port: 80
            initialDelaySeconds: 15
            periodSeconds: 10
```

3. ` kubectl ` kullanarak pod oluşturun
```
$ kubectl apply -f examples/deploy-live.yaml
```
4. ` kubectl `kullanarak pod'u kontrol edin:
```
$ kubectl get pods
```

Pod neden restart aldı?

# StatefulSet
* StatefulSet, durumlu uygulamaları (stateful applications) yönetmek için kullanılan iş yükü (workload) API nesnesidir.

### Alıştırma: YAML'dan StatefulSet Yönetimi

1. Klasörünüzde `statefulset.yaml` oluşturun.

2. Aşağıdaki YAML dosyasını kopyalayın veya `examples/statefulset.yaml` kullanın 

**examples/statefulset.yaml**
```yaml
apiVersion: apps/v1
kind: StatefulSet
metadata:
  name: web
spec:
  selector:
    matchLabels:
      app: nginx 
  serviceName: "nginx"
  replicas: 3 
  minReadySeconds: 10 
  template:
    metadata:
      labels:
        app: nginx 
    spec:
      terminationGracePeriodSeconds: 10
      containers:
      - name: nginx
        image: registry.k8s.io/nginx-slim:0.24
        ports:
        - containerPort: 80
          name: web
        volumeMounts:
        - name: www
          mountPath: /usr/share/nginx/html
  volumeClaimTemplates:
  - metadata:
      name: www
    spec:
      accessModes: [ "ReadWriteOnce" ]
      storageClassName: standard
      resources:
        requests:
          storage: 1Gi
```

3. `kubectl` kullanarak statefulset oluşturun:
```
$ kubectl apply -f examples/statefulset.yaml
```

4. Oluşturulan Pod'ları hemen izleyin:
```
$ kubectl get pods --show-labels --watch
```
* Deployment'ların veya DaemonSet'lerin aksine, bir StatefulSet'in Pod'ları, sıra indekslerine (ordinal index) göre teker teker oluşturulur.

5. StatefulSet hakkında daha fazla bilgi istiyorsanız:
```
$ kubectl describe statefulset statefulset-example
```
6. Mevcut Persistent Volume Claim'leri (PVC) görüntüleyin:
```
$ kubectl get pvc
```
* StatefulSet denetleyicisi (controller), `volumeClaimTemplate`'e dayanarak her örnek (instance) için bir volume oluşturur. Volume adını Pod adının önüne ekler. örn. `www-statefulset-example-0`.
-------

7. StatefulSet'in Pod Şablonunu güncelleyin ve birkaç ek etiket ekleyin:
```
$ kubectl apply -f examples/statefulSet.yaml --record
  < or >
$ kubectl edit statefulset web --record
```
* Pod'ları izlemeye geri dönün:
```
$ kubectl get pods --show-labels
```
* Pod'ların hiçbiri Pod'un yeni sürümüne güncellenmiyor.

8. `web-2` Pod'unu silin:
```
$ kubectl delete pod statefulset-example-2
```
* Hemen Pod'ları alın:
```
$ kubectl get pods --show-labels --watch
```
* Yeni `web-2` Pod'u (veya `web-2`) yeni ek etiketlerle oluşturulmalıdır. `OnDelete` Güncelleme Stratejisi, önceki silinene kadar Pod'un yeni bir iterasyonunu başlatmaz. Bu, StatefulSet için güncelleme sürecinin manuel olarak kapılanmasına (manual gating) olanak tanır.

-----
**Özet**: Bir StatefulSet, diğer herhangi bir Controller (Denetleyici) ile aynı model altında çalışır. İstediğiniz durumu bir StatefulSet nesnesinde tanımlarsınız ve StatefulSet denetleyicisi mevcut durumdan oraya ulaşmak için gerekli güncellemeleri yapar.

## Temizlik (Clean Up)
```
$ kubectl delete -f examples/statefulSet.yaml
$ kubectl delete pvc --all
```

# DaemonSet

* Tüm Node'ların (Düğümlerin) bir Pod kopyasını çalıştırmasını sağlar.
* Cluster'a node'lar eklendikçe, Pod'lar onlara eklenir.
* Cluster'dan node'lar kaldırıldıkça, bu Pod'lar çöp toplanır (garbage collected).
* Bir DaemonSet'i silmek, oluşturduğu Pod'ları temizler.

 ### Alıştırma: YAML'dan DaemonSet Yönetimi

1. Klasörünüzde `daemonset.yaml` oluşturun.

2. Aşağıdaki YAML dosyasını kopyalayın veya `examples/daemonSet.yaml` kullanın

**examples/daemonset.yaml**
```yaml
apiVersion: apps/v1
kind: DaemonSet
metadata:
  name: nginx-daemonset
  namespace: my-namespace
spec:
  selector:
    matchLabels:
      app: nginx
  template:
    metadata:
      labels:
        app: nginx
        role: webserver
        tier: frontend
        environment: production
        owner: myteam
    spec:
      containers:
      - name: nginx
        image: nginx
        resources:
          requests:
            cpu: 50m
            memory: 100Mi
```

3. `kubectl` kullanarak DaemonSet oluşturun:
```
$ kubectl apply -f examples/daemonset.yaml 
```
4. Mevcut DaemonSet'leri görüntüleyin:
```
$ kubectl get daemonset
```
* Şimdi `nginx-daemonset` DaemonSet'inin tek bir örneği (instance) dağıtılmış (deployed) olmalıdır.

5. DaemonSet'i describe etmek (detaylarını göstermek), size cluster genelindeki Dağıtımı (Deployment) ile ilgili durum bilgisi sağlayacaktır.
```
$ kubectl describe ds nginx-daemonset
```
----
----
**Özet**: Basit bir durumda, her tür daemon için tüm node'ları kapsayan bir DaemonSet kullanılır. Daha karmaşık bir kurulum, tek bir daemon türü için birden fazla DaemonSet kullanabilir, ancak farklı donanım türleri için farklı bayraklar (flags) ve/veya farklı bellek ve cpu istekleri (requests) ile.

## Temizlik (Clean Up)
```
$ kubectl delete -f examples/daemonSet.yaml
```

----
# Jobs

* Bir Job, bir veya daha fazla Pod oluşturur ve belirtilen sayıda pod'un başarıyla sonlanmasını (terminate) sağlar. Pod'lar başarıyla tamamlandıkça, Job başarılı tamamlamaları izler. Belirtilen sayıda başarılı tamamlama sayısına ulaşıldığında, görev (yani Job) tamamlanmış olur.

### Alıştırma: YAML'dan Job Oluşturma

**Hedefler:** `kubectl` kullanarak Job'ların nasıl oluşturulacağını öğrenin.

1. Klasörünüzde `jobs.yaml` oluşturun veya `examples/jobs.yaml` kullanın.

2. Aşağıdaki YAML dosyasını kopyalayın 

**examples/jobs.yaml**
```yaml
apiVersion: batch/v1
kind: Job
metadata:
 name: job-example
spec:
 backoffLimit: 4
 completions: 4
 parallelism: 2
 template:
   spec:
     containers:
     - name: hello
       image: alpine:latest
       command: ["/bin/sh", "-c"]
       args: ["echo hello from $HOSTNAME!"]
     restartPolicy: Never   
```

### Jobs Özellikleri
* backoffLimit
* completions
* parallelism
* restartPolicy
----
3. ` kubectl ` kullanarak Job'ları oluşturun:
```
$ kubectl apply -f examples/jobs.yaml
```
4. ` kubectl ` kullanarak Job'ları kontrol edin:

```
$ kubectl get jobs
```

## Temizlik (Clean Up)
```
$ kubectl delete job job-example
```

----
**Özet:** Basit bir durum, bir Pod'u güvenilir bir şekilde tamamlanana kadar çalıştırmak için bir Job nesnesi oluşturmaktır. Job nesnesi, ilk Pod başarısız olursa veya silinirse (örneğin bir node donanım arızası veya node yeniden başlatılması nedeniyle) yeni bir Pod başlatacaktır.

----

# CronJob

* Bir CronJob, zamana dayalı bir programa (schedule) göre Job'lar oluşturur.

### Alıştırma: Bir CronJob Zamanlama

1. Klasörünüzde `cronjobs.yaml` oluşturun.
2. Aşağıdaki YAML dosyasını kopyalayın veya `examples/cronjobs.yaml` kullanın 

**examples/cronjobs.yaml**
```yaml
apiVersion: batch/v1
kind: CronJob
metadata:
 name: cronjob-example
spec:
  schedule: "*/1 * * * *"
  successfulJobsHistoryLimit: 2
  failedJobsHistoryLimit: 1
  jobTemplate:
    spec:
      completions: 4
      parallelism: 2
      template:
        spec:
          containers:
          - name: hello
            image: alpine:latest
            command: ["/bin/sh", "-c"]
            args: ["echo hello from $HOSTNAME!"]
          restartPolicy: Never

```
-----
3. `kubectl` kullanarak cronJob oluşturun:
```
$ kubectl apply -f examples/cronjobs.yaml
```

4. Çalışması için biraz zaman tanıyın ve ardından Job'ları listeleyin:
```
$ kubectl get cronjobs
```
* `<cronjob-adı>-<unix zaman damgası>` formatında adlandırılmış en az bir Job olmalıdır. En eski Job'un zaman damgasını not edin.

5. Birkaç dakika bekleyin ve Job'ları bir kez daha listeleyin:
* En eski Job kaldırılmış olmalıdır. CronJob denetleyicisi (controller), `successfulJobHistoryLimit` ve `failedJobHistoryLimit` özelliklerine göre Job'ları temizleyecektir (purge). Bu durumda, kesinlikle son 2 başarılı Job'u saklamaktadır.

6. `cronjob-example` CronJob'unu describe edin:
```
$ kubectl describe CronJob cronjob-example
```
* Olaylar (events), Job'ların oluşturulması ve silinmesinin kayıtlarını gösterecektir.

7. `cronjob-example` CronJob'unu düzenleyin (edit) ve `Suspend` alanını bulun. Ardından `true` olarak ayarlayın:
```
$ kubectl edit CronJob cronjob-example
```
* Bu, cronjob'un gelecekteki herhangi bir olayı tetiklemesini önleyecektir ve doğrudan CronJob'u silmek zorunda kalmadan başlangıçta bir sorunu gidermek için kullanışlıdır.

8. CronJob'u silin:
```
$ kubectl delete cronjob cronjob-example
```
* CronJob'u silmek, tüm alt Job'ları silecektir. Sorun gidermeye çalışıyorsanız Job'u geçici olarak 'durdurmak' için `Suspend` kullanın.

### CronJob Özellikleri
* schedule
* successfulJobsHistoryLimit
* failedJobHistoryLimit

**Özet**: Cronjob'lar temel olarak zamanlamalı ve hata yönetimi için bazı özel parametrelere sahip Kubernetes job'larıdır. Her Kubernetes job'u bir ila n pod içerir. Bu podlar görev için tanımlanan bir programı çalıştırır ve görev bittiğinde sonlandırılır.

----
