## Services (Servisler)

Kubernetes içindeki Servisler, Pod'ların açığa çıkarılmış iş yüklerine (exposed workloads) erişmenin birleşik yöntemidir. Statik, cluster'a özgü bir IP verilen ve `kube-proxy` aracılığıyla basit yük dengeleme (load-balancing) sağlayan dayanıklı kaynaklardır (Pod'ların aksine).

### Alıştırma : Pod'larla ClusterIP servisi

1) Aşağıdaki koddan etiketlere sahip bir pod oluşturun veya `examples/service-pod.yaml` dosyasını kullanın.

**exmples/service-pod.yaml**
```yaml
apiVersion: v1
kind: Pod
metadata:
  name: pod-labeled-example
  labels:
    app: nginx
spec:
  containers:
  - name: nginx
    image: nginx:latest
    ports:
    - containerPort: 80
---
apiVersion: v1
kind: Service
metadata:
 name: service-example
spec:
 selector:
   app: nginx
 ports:
 - protocol: TCP
   port: 80
   targetPort: 80
```

2) Yaml dosyasından Pod oluşturun. Fark ettiğiniz gibi etiketler, önceki servis örneğinin selector'ı ile aynıdır. Sadece pod'u oluşturarak bu pod'u servise bağlayacağız.

```
$ kubectl apply -f examples/service-pod.yaml
```
3) İstek yapmak için tekrar geçici pod oluşturun:

```
$ kubectl run -i --tty busybox --image=busybox --restart=Never -- sh 
```

4) Servise tekrar GET isteği yapmayı deneyelim.
```
/# wget service-example
```
Çıktı şu şekilde olmalı:

```
Connecting to service-example (10.102.17.158:80)
index.html           100% |******************************************|   612  0:00:00 ETA
```

Artık servis ve etiketler/seçiciler kullanarak nginx pod'una erişebiliriz.

5) Servisi ve busybox'ı silin ancak pod-labeled-example sonraki örnek için kalacaktır.

```
/# exit

$ kubectl delete service service-example
$ kubectl delete pod busybox
```

## Exercise: Load balancing between two pods.

1. İlgili iki pod'a yük dengeleme (load balancing) yapan bir servis yapalım. İki farklı pod oluşturmak için aşağıdaki yaml'ı kullanın.

__examples/two-pod.yaml__
```yaml
apiVersion: v1
kind: Pod
metadata:
  name: pod-1
  labels:
    app: default
spec:
  containers:
  - name: nginx
    image: nginx:latest
    ports:
    - containerPort: 80

---

apiVersion: v1
kind: Pod
metadata:
  name: pod-2
  labels:
    app: default
spec:
  containers:
  - name: apache2
    image: httpd:2.4-alpine
    ports:
    - containerPort: 80
```

**Command**
```
$ kubectl apply -f examples/two-pod.yaml
```

2. Gördüğünüz gibi iki farklı pod var. Biri `nginx` çalıştırıyor, diğeri `httpd` sunucusu çalıştırıyor. Bu iki pod'a yük dengeleme yapan bir servis oluşturalım.

___examples/two-pod-load-balancer.yaml__
```yaml
apiVersion: v1
kind: Service
metadata:
 name: two-pod-load-balancer-service
spec:
  selector:
    app: default
  ports:
  - protocol: TCP
    port: 80
    targetPort: 80
  type: LoadBalancer
```

**Command**
```
$ kubectl apply -f examples/two-pod-load-balancer.yaml
```

3. Çalışır (running) durumda olduklarından emin olmak için pod'ları listeleyin.

```
$ kubectl get pods 
```

__Expected Output__
```
NAME    READY   STATUS    RESTARTS   AGE
pod-1   1/1     Running   0          81s
pod-2   1/1     Running   0          81s
```

4. Servisimizi inceleyelim.

```
$ kubectl get services
```

__Expected Output__

```
two-pod-load-balancer-service   ClusterIP   10.105.208.143   <none>        80/TCP    4s
```

5. Şimdi basit bir geçici pod oluşturarak bu servise bir istek göndereceğiz. ClusterIP kullandığımız için istekleri cluster içinden göndermeliyiz.

```
$ kubectl run test-pod -it --rm --image=busybox:latest sh
```

6. Aşağıdaki komutu busybox'ın sh işleminde çalıştırın.

```
wget http://two-pod-load-balancer-service:80 && cat index.html && rm -rf index.html
```

> Servisin bu farklı pod'lara yük dengeleme yaptığını fark edeceksiniz


7. Clean Up

```
/# exit
$ kubectl delete pod pod-1
$ kubectl delete pod pod-2
$ kubectl delete svc two-pod-load-balancer-service
```

## Ingress

Kümedeki (cluster) servislere dışarıdan erişimi yöneten, genellikle HTTP tabanlı bir API nesnesidir.
Ingress, yük dengeleme (load balancing), SSL sonlandırma (SSL termination) ve isme dayalı sanal barındırma (name-based virtual hosting) sağlayabilir.
Ingress, küme dışından küme içindeki servislere HTTP ve HTTPS rotalarını (routes) açar. Trafik yönlendirme, Ingress kaynağı (resource) üzerinde tanımlanan kurallarla (rules) kontrol edilir.

> **Not:** Ingress Controller ve Ingress Kaynağı farklı kavramlardır. Ingress Kaynağı'nın çalışabilmesi için bir Ingress Controller gereklidir.

### Alıştırma: Ingress Kaynağı Oluşturma

1)  `examples/ingress-pods.yaml` YAML dosyasından veya aşağıdaki YAML kodundan iki farklı pod oluşturun.

    **examples/ingress-pods.yaml**
    ```yaml
    apiVersion: v1
    kind: Pod
    metadata:
      name: pod1
      labels:
        app: pod1
    spec:
      containers:
      - name: whoami
        image: training/whoami 
        ports:
        - containerPort: 80 
    ---
    apiVersion: v1
    kind: Pod
    metadata:
      name: pod2
      labels:
        app: pod2
    spec:
      containers:
      - name: whoami
        image: training/whoami 
        ports:
        - containerPort: 80 
    ```
      **Komut**
    ```bash
    kubectl apply -f examples/ingress-pods.yaml
    ```

2)  Pod'lar için servisleri (services) oluşturun. `examples/ingress-service.yaml` dosyasını veya aşağıdaki kodu kullanın.

    **examples/ingress-service.yaml**
    ```yaml
    kind: Service
    apiVersion: v1
    metadata:
      name: pod1-service
    spec:
      selector:
        app: pod1
      ports:
      - protocol: TCP
        port: 80       
        targetPort: 80 

    ---

    kind: Service
    apiVersion: v1
    metadata:
      name: pod2-service
    spec:
      selector:
        app: pod2
      ports:
      - protocol: TCP
        port: 80       
        targetPort: 80 
    ```

    **Komut**
    ```bash
    kubectl apply -f examples/ingress-service.yaml
    ```

3)  Gelen istekleri (requests) işlemek için `examples/ingress-resource.yaml` dosyasından veya aşağıdaki koddan ingress kaynağını (resource) oluşturun.

    **examples/ingress-resource.yaml**
    ```yaml
    apiVersion: networking.k8s.io/v1
    kind: Ingress
    metadata:
      name: test-ingress
    spec:
      ingressClassName: nginx
      rules:
      - http:
          paths:
          - path: /service1 
            pathType: Prefix 
            backend:
              service:
                name: pod1-service 
                port:
                  number: 80 
          - path: /service2 
            pathType: Prefix 
            backend:
              service:
                name: pod2-service 
                port:
                  number: 80 
    ```

    **Komut**
    ```bash
    kubectl apply -f examples/ingress-resource.yaml
    ```

4)  Ingress'i listeleyin ve (özellikle minikube kullanıyorsanız) `ADDRESS` sütununda bir IP adresi görünene kadar izleyin.

    ```bash
    kubectl get ingress --watch
    ```
5)  `test-ingress`'in adresini (`ADDRESS` sütunundaki IP) kopyalayın ve favori tarayıcınıza yapıştırın. `/` yolunda (path) tanımlı bir rotamız olmadığı için `default-backend - 404` (veya benzeri bir Nginx 404 sayfası) göreceksiniz. Ancak, `<INGRESS_ADRESI>/service1` ve `<INGRESS_ADRESI>/service2` adreslerine gidin. İki farklı servise başarılı bir şekilde yönlendirildiğinizi fark edeceksiniz.

    ```
    http://<INGRESS_ADRESI>/service1
    http://<INGRESS_ADRESI>/service2
    ```

    VEYA Minikube kullanıyorsanız ve `ADDRESS` sütunu boşsa/çalışmıyorsa:

    ```
    http://$(minikube ip)/service1
    http://$(minikube ip)/service2
    ```

> **SSL Notu:** Tarayıcınızda SSL/TLS sertifikası ile ilgili bir güvenlik uyarısı alırsanız (çünkü varsayılan olarak kendinden imzalı bir sertifika kullanılabilir), devam etmeyi seçin (Genellikle "Gelişmiş" > "..." sitesine ilerle (güvenli değil)" gibi seçeneklerle).

    Yanıt verenlerden biri pod1'in (veya ona hizmet veren `whoami` uygulamasının) bilgilerini, diğeri ise pod2'nin bilgilerini gösterecektir.

---

## Temizlik Komutları

Oluşturulan kaynakları silmek için aşağıdaki komutları çalıştırın:

```bash
kubectl delete -f yaml/ingress-service.yaml
kubectl delete -f yaml/ingress-pods.yaml
kubectl delete ingress test-ingress
```